#include "AlivcAudioRender.h"
#include "AlivcPlayerMainDlg.h"


AlivcAudioRender::AlivcAudioRender()
{
    init();
}

AlivcAudioRender::~AlivcAudioRender()
{
    uninit();
}

void AlivcAudioRender::init()
{
    SDL_Init(SDL_INIT_AUDIO);
}

void AlivcAudioRender::uninit()
{
    if (mDevID > 0) {
        SDL_ClearQueuedAudio(mDevID);
        SDL_CloseAudioDevice(mDevID);
        mDevID = 0;
    }
    SDL_QuitSubSystem(SDL_INIT_AUDIO);
}

void AlivcAudioRender::renderData(void *pcmData, int pcmDataLength, int sampleRate, int nbSamples, int channels, int format)
{
    SDL_AudioSpec spec{0};
    spec.freq = sampleRate;
    if (format == AVP_SAMPLE_FMT_S16 || format == AVP_SAMPLE_FMT_S16P) {
        spec.format = AUDIO_S16SYS;
    } else if (format == AVP_SAMPLE_FMT_FLT || format == AVP_SAMPLE_FMT_FLTP) {
        spec.format = AUDIO_F32SYS;
    }
    spec.channels = channels;
    spec.silence = 0;
    spec.samples = nbSamples;
    spec.userdata = nullptr;
    spec.callback = nullptr;
    if (mDevID == 0) {
        if (mAudioDeviceIndex >= 0) {
            int deviceCount = SDL_GetNumAudioDevices(false);
            if (mAudioDeviceIndex < deviceCount) {
                const char *deviceName = SDL_GetAudioDeviceName(mAudioDeviceIndex, false);
                mDevID = SDL_OpenAudioDevice(deviceName, false, &spec, nullptr, 0);
                AlivcLivePlayerMainDlg::LogToFile("[demo] sdl open audio device: %s", deviceName);
            }
        } 
        if (mDevID == 0) {
            mDevID = SDL_OpenAudioDevice(NULL, false, &spec, nullptr, 0);
            AlivcLivePlayerMainDlg::LogToFile("[demo] sdl open default audio device");
        }
        if (mDevID == 0) {
            AlivcLivePlayerMainDlg::LogToFile("[demo] sdl open audio device failed");
            return;
        }
        mSpec = spec;
        SDL_PauseAudioDevice(mDevID, 0);// start play audio
    }
    if (mSpec.freq != spec.freq || mSpec.format != spec.format || mSpec.channels != spec.channels || mSpec.samples != spec.samples) {
        if (mDevID > 0) {
            SDL_CloseAudioDevice(mDevID);
            mDevID = 0;
            if (mAudioDeviceIndex >= 0) {
                int deviceCount = SDL_GetNumAudioDevices(false);
                if (mAudioDeviceIndex < deviceCount) {
                    const char *deviceName = SDL_GetAudioDeviceName(mAudioDeviceIndex, false);
                    mDevID = SDL_OpenAudioDevice(deviceName, false, &spec, nullptr, 0);
                    AlivcLivePlayerMainDlg::LogToFile("[demo] sdl open audio device: %s", deviceName);
                }
            }
            if (mDevID == 0) {
                mDevID = SDL_OpenAudioDevice(NULL, false, &spec, nullptr, 0);
                AlivcLivePlayerMainDlg::LogToFile("[demo] sdl open default audio device");
            }
            if (mDevID == 0) {
                AlivcLivePlayerMainDlg::LogToFile("[demo] sdl open audio device failed");
                return;
            }
            mSpec = spec;
            SDL_PauseAudioDevice(mDevID, 0);// start play audio
        }
    }

    if (mMute) {
        memset(pcmData, 0, pcmDataLength);
    }
    SDL_QueueAudio(mDevID, pcmData, pcmDataLength);
    return;
}

void AlivcAudioRender::setMute(bool mute)
{
    mMute = mute;
}

void AlivcAudioRender::setAudioDevice(int index)
{
    mAudioDeviceIndex = index;
}

void AlivcAudioRender::clearQueue()
{
    if (mDevID > 0) {
        SDL_ClearQueuedAudio(mDevID);
    }
}