#include "AlivcDownloadItemWidget.h"
#include "AlivcNotifyMgr.h"

AlivcDownloadItemWidget::AlivcDownloadItemWidget(QWidget *parent) : QWidget(parent), ui(new Ui::AlivcDownloadItemWidget)
{

    ui->setupUi(this);
    InitEvent();
}

AlivcDownloadItemWidget::~AlivcDownloadItemWidget()
{
    if (mMediaDownloader) {
        mMediaDownloader->stop();
        AliMediaDownloader::DestroyDownloader(mMediaDownloader);
        mMediaDownloader = nullptr;
    }
}

void AlivcDownloadItemWidget::SetSourceInfo(const DownloadSourceInfo &sourceInfo, const QString saveDir)
{
    mSourceInfo = sourceInfo;
    mSaveDir = saveDir;
    Prepare();
}

DownloadSourceInfo AlivcDownloadItemWidget::GetSourceInfo()
{
    return mSourceInfo;
}

void AlivcDownloadItemWidget::SetListItem(QListWidgetItem *listItem)
{
    mListItem = listItem;
}

QListWidgetItem *AlivcDownloadItemWidget::GetListItem()
{
    return mListItem;
}

void AlivcDownloadItemWidget::Start()
{
    if (mDownloaded) {
        return;
    }
    if (mMediaDownloader) {
        ui->labelStatus->setText(QStringLiteral(""));
        mMediaDownloader->start();
    }
}

void AlivcDownloadItemWidget::Delete()
{
    if (mMediaDownloader) {
        ui->labelStatus->setText(QStringLiteral("ļɾ"));
        mMediaDownloader->stop();
        mMediaDownloader->deleteDownloadedFile();
        mDownloaded = false;
        AliMediaDownloader::DestroyDownloader(mMediaDownloader);
        mMediaDownloader = nullptr;
    }
    AlivcNotifyMgr::GetInstance()->NotifyDeleteListItem(mListItem);
}

void AlivcDownloadItemWidget::onPrepared(AliMediaDownloader *downloader, AVPMediaInfo *mediaInfo)
{
    emit SgnPrepared(new AVPMediaInfo(*mediaInfo));
}

void AlivcDownloadItemWidget::onError(AliMediaDownloader *downloader, AVPErrorModel *errorModel)
{
    QString error = QString::fromLocal8Bit(errorModel->message);
    emit SgnError(error);
}

void AlivcDownloadItemWidget::onDownloadingProgress(AliMediaDownloader *downloader, int percent)
{
    emit SgnDownloadingProgress(percent);
}

void AlivcDownloadItemWidget::onProcessingProgress(AliMediaDownloader *downloader, int percent)
{
    emit SgnProcessingProgress(percent);
}

void AlivcDownloadItemWidget::onCompletion(AliMediaDownloader *downloader)
{
    emit SgnCompletion();
}

void AlivcDownloadItemWidget::on_btnStart_clicked()
{
    Start();
}

void AlivcDownloadItemWidget::on_btnStop_clicked()
{
    if (mMediaDownloader) {
        ui->labelStatus->setText(QStringLiteral("ֹͣ"));
        mMediaDownloader->stop();
    }
}

void AlivcDownloadItemWidget::on_btnPlay_clicked()
{
    if (!mDownloaded) {
        return;
    }
    if (mMediaDownloader) {
        QString filePath = QString::fromLocal8Bit(mMediaDownloader->getDownloadedFilePath());
        AlivcNotifyMgr::GetInstance()->NotifyPlayWithUrl(filePath);
    }
}

void AlivcDownloadItemWidget::on_btnDelete_clicked()
{
    Delete();
}

void AlivcDownloadItemWidget::OnPreparedUI(AVPMediaInfo *mediaInfo)
{
    if (mMediaDownloader) {
        mMediaDownloader->setSaveDirectory(mSaveDir.toLocal8Bit());
        mMediaDownloader->selectTrack(mSourceInfo.trackIndex);
    }
    QString coverUrl = QString::fromStdString(mediaInfo->coverURL);
    QString title = QString::fromStdString(mediaInfo->title);
    ui->labelStatus->setText(QStringLiteral("׼"));
    ui->labelName->setText(mSourceInfo.trackDesc + "|" + title);
    delete mediaInfo;
}

void AlivcDownloadItemWidget::OnErrorUI(QString error)
{
    ui->labelStatus->setText(QStringLiteral("") + error);
}

void AlivcDownloadItemWidget::OnDownloadingProgressUI(int percent)
{
    ui->labelStatus->setText(QStringLiteral(":%1%").arg(percent));
}

void AlivcDownloadItemWidget::OnProcessingProgressUI(int percent)
{
    ui->labelStatus->setText(QStringLiteral("ļ:%1%").arg(percent));
}

void AlivcDownloadItemWidget::OnCompletionUI()
{
    ui->labelStatus->setText(QStringLiteral(""));
    mDownloaded = true;
}

void AlivcDownloadItemWidget::InitEvent()
{
    connect(this, &AlivcDownloadItemWidget::SgnPrepared, this, &AlivcDownloadItemWidget::OnPreparedUI);
    connect(this, &AlivcDownloadItemWidget::SgnError, this, &AlivcDownloadItemWidget::OnErrorUI);
    connect(this, &AlivcDownloadItemWidget::SgnDownloadingProgress, this, &AlivcDownloadItemWidget::OnDownloadingProgressUI);
    connect(this, &AlivcDownloadItemWidget::SgnProcessingProgress, this, &AlivcDownloadItemWidget::OnProcessingProgressUI);
    connect(this, &AlivcDownloadItemWidget::SgnCompletion, this, &AlivcDownloadItemWidget::OnCompletionUI);
}

void AlivcDownloadItemWidget::Prepare()
{
    if (mMediaDownloader == nullptr) {
        mMediaDownloader = AliMediaDownloader::CreateMediaDownloader();
        mMediaDownloader->setListener(this);
    }
    if (mSourceInfo.sourceType == DownloadSourceTypeSts) {
        AVPVidStsSource stsSource;
        stsSource.initWithVid(mSourceInfo.vid.toLocal8Bit(), mSourceInfo.accessKeyId.toLocal8Bit(),
                              mSourceInfo.accessKeySecret.toLocal8Bit(), mSourceInfo.securityToken.toLocal8Bit(),
                              mSourceInfo.region.toLocal8Bit(), nullptr);
        mMediaDownloader->prepareWithVid(&stsSource);
    } else if (mSourceInfo.sourceType == DownloadSourceTypePlayAuth) {
        AVPVidAuthSource authSource;
        authSource.initWithVid(mSourceInfo.vid.toLocal8Bit(), mSourceInfo.playAuth.toLocal8Bit(), mSourceInfo.region.toLocal8Bit(),
                               nullptr);
        mMediaDownloader->prepareWithPlayAuth(&authSource);
    } else {
        assert(false);
    }
}