#include "AlivcDownloaderDlg.h"
#include "AlivcDownloadItemWidget.h"
#include "AlivcDownloadPrepareDlg.h"
#include "AlivcNotifyMgr.h"
#include "QFileDialog"
#include "QMessageBox"

AlivcDownloaderDlg::AlivcDownloaderDlg(QWidget *parent) : QDialog(parent), ui(new Ui::AlivcDownloaderDlg)
{

    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
    ui->setupUi(this);
    connect(AlivcNotifyMgr::GetInstance(), &AlivcNotifyMgr::SngDeleteListItem, this, &AlivcDownloaderDlg::OnDeleteListItem);
}

void AlivcDownloaderDlg::closeEvent(QCloseEvent *event)
{
    event->ignore();
    hide();
}

void AlivcDownloaderDlg::on_btnDone_clicked()
{
    hide();
}

void AlivcDownloaderDlg::on_btnSelectFolder_clicked()
{
    QString path = QFileDialog::getExistingDirectory(this, QStringLiteral("ѡ񱣴Ŀ¼"), "./",
                                                     QFileDialog::ShowDirsOnly | QFileDialog::DontUseNativeDialog);
    ui->editSaveFolder->setText(path);
}

void AlivcDownloaderDlg::on_btnAddDownloadItem_clicked()
{
    static AlivcDownloadPrepareDlg dlg(this);
    dlg.CheckAuthInfo();
    int ret = dlg.exec();
    if (ret == QDialog::Accepted) {
        DownloadSourceInfo sourceInfo = dlg.GetDownloadSourceInfo();
        // if the download item exist, do not add
        int count = ui->listDownloadItem->count();
        for (int i = 0; i < count; i++) {
            QListWidgetItem *listItem = ui->listDownloadItem->item(i);
            AlivcDownloadItemWidget *itemWidget = (AlivcDownloadItemWidget *) ui->listDownloadItem->itemWidget(listItem);
            DownloadSourceInfo existSourceInfo = itemWidget->GetSourceInfo();
            if (existSourceInfo.vid == sourceInfo.vid && existSourceInfo.trackIndex == sourceInfo.trackIndex) {
                QMessageBox::warning(this, QStringLiteral(""), QStringLiteral("Ѵ"), QMessageBox::Yes);
                return;
            }
        }
        //add download item
        AlivcDownloadItemWidget *itemWidget = new AlivcDownloadItemWidget(ui->listDownloadItem);
        itemWidget->SetSourceInfo(sourceInfo, ui->editSaveFolder->text());
        QListWidgetItem *listItem = new QListWidgetItem();
        itemWidget->SetListItem(listItem);
        ui->listDownloadItem->addItem(listItem);
        listItem->setSizeHint(QSize(0, 112));
        ui->listDownloadItem->setItemWidget(listItem, itemWidget);
        ui->labelCurrentCount->setText(QStringLiteral("ǰ") + QString::number(ui->listDownloadItem->count()));
    }
}

void AlivcDownloaderDlg::on_btnDownloadAll_clicked()
{
    int count = ui->listDownloadItem->count();
    for (int i = 0; i < count; i++) {
        QListWidgetItem *listItem = ui->listDownloadItem->item(i);
        AlivcDownloadItemWidget *itemWidget = (AlivcDownloadItemWidget *) ui->listDownloadItem->itemWidget(listItem);
        itemWidget->Start();
    }
}

void AlivcDownloaderDlg::on_btnDeleteAll_clicked()
{
    int count = ui->listDownloadItem->count();
    for (int i = count - 1; i >= 0; i--) {
        QListWidgetItem *listItem = ui->listDownloadItem->item(i);
        AlivcDownloadItemWidget *itemWidget = (AlivcDownloadItemWidget *) ui->listDownloadItem->itemWidget(listItem);
        itemWidget->Delete();
    }
}

void AlivcDownloaderDlg::OnDeleteListItem(QListWidgetItem *listItem)
{
    if (ui->listDownloadItem) {
        ui->listDownloadItem->takeItem(ui->listDownloadItem->row(listItem));
        ui->labelCurrentCount->setText(QStringLiteral("ǰ") + QString::number(ui->listDownloadItem->count()));
    }
}