#include "AlivcPlayerConfigDlg.h"
#include <QHBoxLayout>
#include <QVBoxLayout>

AlivcPlayerConfigDlg::AlivcPlayerConfigDlg(QWidget *parent, AVPConfig *config) : QDialog(parent)
{
    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
    if (config) {
        mAvConfig = *config;
    }
    InitLayout();
    InitEvent();
    InitData();
}

AlivcPlayerConfigDlg::~AlivcPlayerConfigDlg()
{}

void AlivcPlayerConfigDlg::InitLayout()
{
    QVBoxLayout *mainLayout = new QVBoxLayout();
    mainLayout->setContentsMargins(20, 40, 20, 20);

    mRefererEdit = new QLineEdit();
    QHBoxLayout *referLayout = AddConfigItem(QStringLiteral("Refer"), mRefererEdit);
    mainLayout->addLayout(referLayout);
    if (mAvConfig.referer) {
        QString str = QString::fromStdString(mAvConfig.referer);
        mRefererEdit->setText(str);
    }

    mHttpProxyEdit = new QLineEdit();
    QHBoxLayout *httpProxyLayout = AddConfigItem(QStringLiteral("HttpProxy"), mHttpProxyEdit);
    mainLayout->addLayout(httpProxyLayout);
    if (mAvConfig.httpProxy) {
        QString str = QString::fromStdString(mAvConfig.httpProxy);
        mHttpProxyEdit->setText(str);
    }

    mUserAgentEdit = new QLineEdit();
    QHBoxLayout *userAgentLayout = AddConfigItem(QStringLiteral("UserAgent"), mUserAgentEdit);
    mainLayout->addLayout(userAgentLayout);
    if (mAvConfig.userAgent) {
        QString str = QString::fromStdString(mAvConfig.userAgent);
        mUserAgentEdit->setText(str);
    }

    mNetworkTimeoutEdit = new QLineEdit();
    QHBoxLayout *networkTimeoutLayout = AddConfigItem(QStringLiteral("NetworkTimeout"), mNetworkTimeoutEdit);
    mainLayout->addLayout(networkTimeoutLayout);
    AddNumberVaild(mNetworkTimeoutEdit);
    QString s = QString::number(mAvConfig.networkTimeout);
    mNetworkTimeoutEdit->setText(s);

    mNetworkRetryCountEdit = new QLineEdit();
    QHBoxLayout *networkRetryCountLayout = AddConfigItem(QStringLiteral("NetworkRetryCount"), mNetworkRetryCountEdit);
    mainLayout->addLayout(networkRetryCountLayout);
    AddNumberVaild(mNetworkRetryCountEdit);
    s = QString::number(mAvConfig.networkRetryCount);
    mNetworkRetryCountEdit->setText(s);

    mMaxBufferDurationEdit = new QLineEdit();
    QHBoxLayout *maxBufferDurationLayout = AddConfigItem(QStringLiteral("Max Buffer Duration"), mMaxBufferDurationEdit);
    mainLayout->addLayout(maxBufferDurationLayout);
    AddNumberVaild(mMaxBufferDurationEdit);
    s = QString::number(mAvConfig.maxBufferDuration);
    mMaxBufferDurationEdit->setText(s);

    mStartBufferDurationEdit = new QLineEdit();
    QHBoxLayout *startBufferDurationLayout = AddConfigItem(QStringLiteral("Start Buffer Duration"), mStartBufferDurationEdit);
    mainLayout->addLayout(startBufferDurationLayout);
    AddNumberVaild(mStartBufferDurationEdit);
    s = QString::number(mAvConfig.startBufferDuration);
    mStartBufferDurationEdit->setText(s);

    mHighBufferDurationEdit = new QLineEdit();
    QHBoxLayout *highBufferDurationLayout = AddConfigItem(QStringLiteral("High Buffer Duration"), mHighBufferDurationEdit);
    mainLayout->addLayout(highBufferDurationLayout);
    AddNumberVaild(mHighBufferDurationEdit);
    s = QString::number(mAvConfig.highBufferDuration);
    mHighBufferDurationEdit->setText(s);

    mMaxDelayTimeEdit = new QLineEdit();
    QHBoxLayout *maxDelayTimeLayout = AddConfigItem(QStringLiteral("Max Delay Time"), mMaxDelayTimeEdit);
    mainLayout->addLayout(maxDelayTimeLayout);
    AddNumberVaild(mMaxDelayTimeEdit);
    s = QString::number(mAvConfig.maxDelayTime);
    mMaxDelayTimeEdit->setText(s);

    mHttpHeadersEdit = new QLineEdit();
    QHBoxLayout *httpHeadersLayout = AddConfigItem(QStringLiteral("HTTP headers"), mHttpHeadersEdit);
    mainLayout->addLayout(httpHeadersLayout);
    s = "";
    if (mAvConfig.headerCount > 0) {
        for (int i = 0; i < mAvConfig.headerCount; i++) {
            if (i != 0) {
                s.append(";");
            }
            s.append(mAvConfig.httpHeaders[i]);
        }
    }
    mHttpHeadersEdit->setText(s);

    mShowLastFrameWhenStop = new QCheckBox();
    QHBoxLayout *showLastFrameWhenStopLayout = AddConfigItem(QStringLiteral("Show Last Frame When Stop"), mShowLastFrameWhenStop);
    showLastFrameWhenStopLayout->addStretch();
    mainLayout->addLayout(showLastFrameWhenStopLayout);
    mShowLastFrameWhenStop->setChecked(mAvConfig.showLastFrameWhenStop);

    mainLayout->addStretch();

    mConfigButton = new QPushButton(QStringLiteral(""));
    QHBoxLayout *buttonLayout = new QHBoxLayout();
    buttonLayout->addStretch();
    buttonLayout->addWidget(mConfigButton);
    buttonLayout->addStretch();
    mainLayout->addLayout(buttonLayout);

    setLayout(mainLayout);
}

void AlivcPlayerConfigDlg::InitEvent()
{
    connect(mConfigButton, &QPushButton::clicked, this, &AlivcPlayerConfigDlg::OnConfigButtonClicked);
}

void AlivcPlayerConfigDlg::InitData()
{}

QHBoxLayout *AlivcPlayerConfigDlg::AddConfigItem(QString configName, QWidget *editWidget)
{
    QHBoxLayout *configLayout = new QHBoxLayout;
    QLabel *configLabel = new QLabel(configName);
    configLabel->setMinimumWidth(200);
    QSizePolicy sizePolicy = editWidget->sizePolicy();
    sizePolicy.setHorizontalPolicy(QSizePolicy::Expanding);
    sizePolicy.setVerticalPolicy(QSizePolicy::Minimum);
    configLayout->addWidget(configLabel);
    configLayout->addWidget(editWidget);
    configLayout->setMargin(0);
    configLayout->setSpacing(10);
    return configLayout;
}

void AlivcPlayerConfigDlg::AddNumberVaild(QLineEdit *edit)
{
    if (edit == nullptr) {
        return;
    }

    QRegExp regx("[0-9]+$");
    QValidator *validator = new QRegExpValidator(regx, edit);
    edit->setValidator(validator);
}

void AlivcPlayerConfigDlg::OnConfigButtonClicked(bool checked)
{
    mAvConfig = AVPConfig();
    mAvConfig.highBufferDuration = mHighBufferDurationEdit->text().toInt();
    mAvConfig.httpProxy = _strdup(mHttpProxyEdit->text().toStdString().c_str());
    mAvConfig.userAgent = _strdup(mUserAgentEdit->text().toStdString().c_str());
    mAvConfig.maxBufferDuration = mMaxBufferDurationEdit->text().toInt();
    mAvConfig.maxDelayTime = mMaxDelayTimeEdit->text().toInt();
    mAvConfig.networkTimeout = mNetworkTimeoutEdit->text().toInt();
    mAvConfig.networkRetryCount = mNetworkRetryCountEdit->text().toInt();
    mAvConfig.referer = _strdup(mRefererEdit->text().toStdString().c_str());
    mAvConfig.showLastFrameWhenStop = mShowLastFrameWhenStop->isChecked();
    mAvConfig.startBufferDuration = mStartBufferDurationEdit->text().toInt();
    //HTTP header
    QStringList list = mHttpHeadersEdit->text().split(";", QString::SkipEmptyParts);
    mAvConfig.headerCount = list.size();
    mAvConfig.httpHeaders = new char *[mAvConfig.headerCount];
    for (int i = 0; i < mAvConfig.headerCount; i++) {
        mAvConfig.httpHeaders[i] = _strdup(list[i].toStdString().c_str());
    }
    accept();
}
