#include "AlivcVideoRender.h"

AlivcVideoRender::AlivcVideoRender(HWND hwnd)
{
    init(hwnd);
}

AlivcVideoRender::~AlivcVideoRender()
{
    uninit();
}

void AlivcVideoRender::init(HWND hwnd)
{
    SDL_InitSubSystem(SDL_INIT_VIDEO);
    mVideoWindow = SDL_CreateWindowFrom(hwnd);
    SDL_ShowWindow(mVideoWindow);
    SDL_SetHint(SDL_HINT_RENDER_DRIVER, "direct3d11");
    mVideoRender = SDL_CreateRenderer(mVideoWindow, -1, SDL_RENDERER_SOFTWARE);
    int renderWidth = 0;
    int renderHeight = 0;
    SDL_GL_GetDrawableSize(mVideoWindow, &mWindowWidth, &mWindowHeight);
    SDL_GetRendererOutputSize(mVideoRender, &renderWidth, &renderHeight);
    float DISPLAY_WIDTH_RATIO = renderWidth * 1.0f / mWindowWidth;
    float DISPLAY_HEIGHT_RATIO = renderHeight * 1.0f / mWindowHeight;
    SDL_RenderSetScale(mVideoRender, DISPLAY_WIDTH_RATIO, DISPLAY_HEIGHT_RATIO);
    SDL_setenv(SDL_HINT_RENDER_SCALE_QUALITY, "linear", 0);
}

void AlivcVideoRender::uninit()
{
    if (mVideoTexture) {
        SDL_DestroyTexture(mVideoTexture);
        mVideoTexture = nullptr;
    }
    if (mVideoRender ) {
        SDL_DestroyRenderer(mVideoRender);
        mVideoRender = nullptr;
    }
    if (mVideoWindow) {
        SDL_DestroyWindow(mVideoWindow);
        mVideoWindow = nullptr;
    }
    SDL_QuitSubSystem(SDL_INIT_VIDEO);
}

void AlivcVideoRender::renderData(void **yuvData, int *lineSize, int width, int height)
{
    recreateTextureIfNeed(width, height);
    SDL_Rect srcRect{};
    srcRect.x = 0;
    srcRect.y = 0;
    srcRect.w = mVideoWidth;
    srcRect.h = mVideoHeight;
    int angle = 0;
    SDL_RendererFlip flip = SDL_FLIP_NONE;
    SDL_Rect dstRect = getDestRet();
    uint8_t **data = (uint8_t **) yuvData;
    if (mVideoRender != nullptr && mVideoTexture != nullptr) {
        SDL_UpdateYUVTexture(mVideoTexture, &srcRect, data[0], lineSize[0], data[1], lineSize[1], data[2], lineSize[2]);
        SDL_RenderClear(mVideoRender);
        SDL_RenderCopyEx(mVideoRender, //SDL_Renderer*          renderer,
                         mVideoTexture,//SDL_Texture*           texture,
                         &srcRect,     //const SDL_Rect*        srcrect,
                         &dstRect,     //const SDL_Rect*        dstrect,
                         angle,        //const double           angle,
                         nullptr,      //const SDL_Point*       center,
                         flip          //const SDL_RendererFlip flip
        );
        SDL_RenderPresent(mVideoRender);
    }
}

void AlivcVideoRender::recreateTextureIfNeed(int videoWidth, int videoHeight)
{
    if (mInited && videoHeight == mVideoHeight && videoWidth == mVideoWidth) {
        return;
    }
    mInited = true;
    if (mVideoTexture != nullptr) {
        SDL_DestroyTexture(mVideoTexture);
        mVideoTexture = nullptr;
    }
    if (mVideoTexture == nullptr) {
        mVideoTexture = SDL_CreateTexture(mVideoRender, SDL_PIXELFORMAT_IYUV, SDL_TEXTUREACCESS_STREAMING, videoWidth, videoHeight);
    }
    if (mVideoTexture == nullptr) {
        return;
    }
    mVideoWidth = videoWidth;
    mVideoHeight = videoHeight;
}

SDL_Rect AlivcVideoRender::getDestRet()
{
    SDL_Rect dstRect{};
    SDL_GL_GetDrawableSize(mVideoWindow, &mWindowWidth, &mWindowHeight);
    if (mWindowWidth == 0 || mWindowHeight == 0 || mVideoWidth == 0 || mVideoHeight == 0) {
        dstRect.x = 0;
        dstRect.y = 0;
        dstRect.w = 0;
        dstRect.h = 0;
        return dstRect;
    }

    float videoWhRatio = 1.0f * mVideoWidth / mVideoHeight;
    float windowWhRatio = 1.0f * mWindowWidth / mWindowHeight;

    if (videoWhRatio <= windowWhRatio) {
        dstRect.h = mWindowHeight;
        dstRect.w = static_cast<int>(dstRect.h * videoWhRatio);
    } else {
        dstRect.w = mWindowWidth;
        dstRect.h = static_cast<int>(dstRect.w / videoWhRatio);
    }

    dstRect.y = (mWindowHeight - dstRect.h) / 2;
    dstRect.x = (mWindowWidth - dstRect.w) / 2;
    return dstRect;
}