﻿//
//  AVPDef.h
//
//  Created by shiping.csp on 2018/11/16.
//  Copyright © 2018 com.alibaba.AliyunPlayer. All rights reserved.
//

#ifndef AVPDef_h
#define AVPDef_h

#ifdef WIN32
#ifdef ALIVC_STATIC
#define ALIVC_EXTERN
#else
/* Windows - set up .dll .so import/export decorators. */
#ifdef BUILD_ALIVC_DLL
/* Building library. */
#define ALIVC_EXTERN __declspec(dllexport)
#else
#define ALIVC_EXTERN __declspec(dllimport)
#endif
#endif

#ifdef ALIVC_STATIC
#define ALIVC_EXTERN
#else
/* Windows - set up .dll .so import/export decorators. */
#ifdef BUILD_ALIVC_PLUGIN_DLL
/* Building library. */
#define ALIVC_PLUGIN_EXTERN __declspec(dllexport)
#else
#define ALIVC_PLUGIN_EXTERN __declspec(dllimport)
#endif
#endif

#else
#define ALIVC_PLUGIN_EXTERN
#define ALIVC_EXTERN __attribute__((visibility("default")))
//#ifdef __GNUC__
//#define ALIVC_EXTERN __attribute__ ((visibility("default")))
//#else
//#define ALIVC_EXTERN __attribute__ ((visibility("default")))
//#endif
#endif

#include "AVPErrorCode.h"

namespace alivc_player {

    /**
    * Convert playback URL before playback.
    *
    * For vid playback, try to convert the playback URL before playback
    *
    * @param   srcURL  [in]  input URL.
    * @param   srcFormat [in]  input format. should be "m3u8" "mp4", or "" if unknown.
    * @param   destURL [out] output URL, convertURL function will malloc the memory, and user of PlayURLConverCallback need free it outside.
    *
    * @return  true if success.
    */
    typedef bool (*PlayURLConvertCallback)(const char *srcURL, const char *srcFormat, char **destURL);

    typedef enum _AVPStatus {
        AVPStatusIdle = 0,
        AVPStatusInitialzed,
        AVPStatusPrepared,
        AVPStatusStarted,
        AVPStatusPaused,
        AVPStatusStopped,
        AVPStatusCompletion,
        AVPStatusError
    } AVPStatus;

    /**@brief 跳转模式，是否为精准跳转*/
    /****@brief Seeking mode: accurate seeking or inaccurate seeking.*/
    typedef enum _AVPSeekMode {
        AVP_SEEKMODE_ACCURATE = 0x01,
        AVP_SEEKMODE_INACCURATE = 0x10,
    } AVPSeekMode;

    /**@brief 渲染显示模式*/
    /****@brief Zoom mode*/
    typedef enum _AVPScalingMode {
        /**@brief 不保持比例平铺*/
        /****@brief Auto stretch to fit.*/
        AVP_SCALINGMODE_SCALETOFILL,
        /**@brief 保持比例，黑边*/
        /****@brief Keep aspect ratio and add black borders.*/
        AVP_SCALINGMODE_SCALEASPECTFIT,
        /**@brief 保持比例填充，需裁剪*/
        /****@brief Keep aspect ratio and crop.*/
        AVP_SCALINGMODE_SCALEASPECTFILL,
    } AVPScalingMode;

    /**@brief 旋转模式*/
    /****@brief Rotate mode*/
    typedef enum AVPRotateMode { AVP_ROTATE_0 = 0, AVP_ROTATE_90 = 90, AVP_ROTATE_180 = 180, AVP_ROTATE_270 = 270 } AVPRotateMode;

    /**@brief 镜像模式*/
    /****@brief Mirroring mode*/
    typedef enum _AVPMirrorMode {
        AVP_MIRRORMODE_NONE,
        AVP_MIRRORMODE_HORIZONTAL,
        AVP_MIRRORMODE_VERTICAL,
    } AVPMirrorMode;

    /**@brief 播放器事件类型*/
    /****@brief Player event type*/
    typedef enum _AVPEventType {
        /**@brief 准备完成事件*/
        /****@brief Preparation completion event*/
        AVPEventPrepareDone,
        /**@brief 自动启播事件*/
        /****@brief Autoplay start event*/
        AVPEventAutoPlayStart,
        /**@brief 首帧显示事件*/
        /****@brief First frame display event*/
        AVPEventFirstRenderedStart,
        /**@brief 播放完成事件*/
        /****@brief Playback completion event*/
        AVPEventCompletion,
        /**@brief 缓冲开始事件*/
        /****@brief Buffer start event*/
        AVPEventLoadingStart,
        /**@brief 缓冲完成事件*/
        /****@brief Buffer completion event*/
        AVPEventLoadingEnd,
        /**@brief 跳转完成事件*/
        /****@brief Seeking completion event*/
        AVPEventSeekEnd,
        /**@brief 循环播放开始事件*/
        /****@brief Loop playback start event*/
        AVPEventLoopingStart,
    } AVPEventType;

    typedef enum _AVPLogLevel {
        LOG_LEVEL_NONE = 0,
        LOG_LEVEL_FATAL = 8,
        LOG_LEVEL_ERROR = 16,
        LOG_LEVEL_WARNING = 24,
        LOG_LEVEL_INFO = 32,
        LOG_LEVEL_DEBUG = 48,
        LOG_LEVEL_TRACE = 56,
    } AVPLogLevel;

    typedef struct _AVPErrorModel {
        AVPErrorCode code;
        char *message = nullptr;
        char *requestId = nullptr;
        char *videoId = nullptr;
    } AVPErrorModel;
};// namespace alivc_player

#endif /* AVPDef_h */
