﻿//
//  AVPFrameInfo.h
//
//  Created by yuyuan on 2021/05/21.
//

#ifndef AVPFrameInfo_h
#define AVPFrameInfo_h

#include "AVPDef.h"
#include <stdint.h>

namespace alivc_player {

    enum AVPFrameType {
        AVP_FrameType_Unknown,
        AVP_FrameType_Video,
        AVP_FrameType_Audio
    };

    enum AVPPixelFormat {
        AVP_PIX_FMT_NONE = -1,
        AVP_PIX_FMT_YUV420P,  ///< planar YUV 4:2:0, 12bpp, (1 Cr & Cb sample per 2x2 Y samples)
        AVP_PIX_FMT_YUYV422,  ///< packed YUV 4:2:2, 16bpp, Y0 Cb Y1 Cr
        AVP_PIX_FMT_RGB24,    ///< packed RGB 8:8:8, 24bpp, RGBRGB...
        AVP_PIX_FMT_BGR24,    ///< packed RGB 8:8:8, 24bpp, BGRBGR...
        AVP_PIX_FMT_YUV422P,  ///< planar YUV 4:2:2, 16bpp, (1 Cr & Cb sample per 2x1 Y samples)
        AVP_PIX_FMT_YUV444P,  ///< planar YUV 4:4:4, 24bpp, (1 Cr & Cb sample per 1x1 Y samples)
        AVP_PIX_FMT_YUV410P,  ///< planar YUV 4:1:0,  9bpp, (1 Cr & Cb sample per 4x4 Y samples)
        AVP_PIX_FMT_YUV411P,  ///< planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples)
        AVP_PIX_FMT_GRAY8,    ///<        Y        ,  8bpp
        AVP_PIX_FMT_MONOWHITE,///<        Y        ,  1bpp, 0 is white, 1 is black, in each byte pixels are ordered from the msb to the lsb
        AVP_PIX_FMT_MONOBLACK,///<        Y        ,  1bpp, 0 is black, 1 is white, in each byte pixels are ordered from the msb to the lsb
        AVP_PIX_FMT_PAL8,     ///< 8 bits with AV_PIX_FMT_RGB32 palette
        AVP_PIX_FMT_YUVJ420P,///< planar YUV 4:2:0, 12bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV420P and setting color_range
        AVP_PIX_FMT_YUVJ422P,///< planar YUV 4:2:2, 16bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV422P and setting color_range
        AVP_PIX_FMT_YUVJ444P,///< planar YUV 4:4:4, 24bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV444P and setting color_range
        AVP_PIX_FMT_UYVY422,  ///< packed YUV 4:2:2, 16bpp, Cb Y0 Cr Y1
        AVP_PIX_FMT_UYYVYY411,///< packed YUV 4:1:1, 12bpp, Cb Y0 Y1 Cr Y2 Y3
        AVP_PIX_FMT_BGR8,     ///< packed RGB 3:3:2,  8bpp, (msb)2B 3G 3R(lsb)
        AVP_PIX_FMT_BGR4,///< packed RGB 1:2:1 bitstream,  4bpp, (msb)1B 2G 1R(lsb), a byte contains two pixels, the first pixel in the byte is the one composed by the 4 msb bits
        AVP_PIX_FMT_BGR4_BYTE,///< packed RGB 1:2:1,  8bpp, (msb)1B 2G 1R(lsb)
        AVP_PIX_FMT_RGB8,     ///< packed RGB 3:3:2,  8bpp, (msb)2R 3G 3B(lsb)
        AVP_PIX_FMT_RGB4,///< packed RGB 1:2:1 bitstream,  4bpp, (msb)1R 2G 1B(lsb), a byte contains two pixels, the first pixel in the byte is the one composed by the 4 msb bits
        AVP_PIX_FMT_RGB4_BYTE,///< packed RGB 1:2:1,  8bpp, (msb)1R 2G 1B(lsb)
        AVP_PIX_FMT_NV12,///< planar YUV 4:2:0, 12bpp, 1 plane for Y and 1 plane for the UV components, which are interleaved (first byte U and the following byte V)
        AVP_PIX_FMT_NV21,///< as above, but U and V bytes are swapped

        AVP_PIX_FMT_YUV420P10BE = 63,///< planar YUV 4:2:0, 15bpp, (1 Cr & Cb sample per 2x2 Y samples), big-endian
        AVP_PIX_FMT_YUV420P10LE,     ///< planar YUV 4:2:0, 15bpp, (1 Cr & Cb sample per 2x2 Y samples), little-endian

        AVP_PIX_FMT_D3D11 = 900,
        AVP_PIX_FMT_DXVA2_VLD,

        AVP_PIX_FMT_APPLE_PIXEL_BUFFER = 1000,
        AVP_PIX_FMT_CICADA_AF,         //framework VideoFrame
        AVP_PIX_FMT_CICADA_MEDIA_CODEC,//Android mediacodec buffer index
    };

    enum AVPSampleFormat {
        AVP_SAMPLE_FMT_NONE = -1,
        AVP_SAMPLE_FMT_U8, ///< unsigned 8 bits
        AVP_SAMPLE_FMT_S16,///< signed 16 bits
        AVP_SAMPLE_FMT_S32,///< signed 32 bits
        AVP_SAMPLE_FMT_FLT,///< float
        AVP_SAMPLE_FMT_DBL,///< double

        AVP_SAMPLE_FMT_U8P, ///< unsigned 8 bits, planar
        AVP_SAMPLE_FMT_S16P,///< signed 16 bits, planar
        AVP_SAMPLE_FMT_S32P,///< signed 32 bits, planar
        AVP_SAMPLE_FMT_FLTP,///< float, planar
        AVP_SAMPLE_FMT_DBLP,///< double, planar

        AVP_SAMPLE_FMT_NB///< Number of sample formats. DO NOT USE if linking dynamically
    };

    typedef struct ALIVC_EXTERN _AVPFrameInfo {
        AVPFrameType frameType = AVP_FrameType_Unknown;
        int64_t pts = 0;
        int64_t duration = 0;
        bool key;
        int64_t timePosition;

        AVPSampleFormat audio_format;
        int audio_nb_samples;
        int audio_channels;
        int audio_sample_rate;
        uint64_t audio_channel_layout;
        int audio_data_lineNum;
        uint8_t **audio_data;
        int audio_data_lineSize;

        AVPPixelFormat video_format;
        int video_width;
        int video_height;
        int video_rotate;
        double video_dar;
        size_t video_crop_top;
        size_t video_crop_bottom;
        size_t video_crop_left;
        size_t video_crop_right;
        int video_colorRange;
        int video_colorSpace;
        int video_data_lineNum;
        int *video_data_lineSize;
        uint8_t **video_data_addr;

    } AVPFrameInfo;

};// namespace alivc_player

#endif /* AVPFrameInfo_h */
